/*
Turing Machine Simulator
Copyright (C) 2005-2011 Samuel BWPOW Kupka, http://bwpow.eu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <cstdio>
#include <cstring>
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include "ts.h"

using namespace std;

extern map<INSTR,INSTR> instr;
extern vector<string> state;
extern vector<char> pr,pl;
extern INFO info;
extern char chars[256];

void save_zeros(FILE *s,int count)
{
    for(int i=0;i<count;i++){
        fputc(info.zero,s);
    }
}

int save_output(void)
{
    FILE *s;
    s=fopen(info.outfile,"w");
    if(s==0) return -1;

    if(info.output&OUTPUT_CONF){
        if(info.cr){
            for(int i=0;i<info.maxmemory;i++){
                if(i<(int)pr.size()) fputc(pr[i],s);
                else fputc(info.zero,s);
            }
        }
        else{
            bool seenOne=false;
            int zeroCount=0;
            if(!info.cl){
                for(int i=pl.size()-1;i>=0;i--){
                    if(pl[i]==info.zero) zeroCount++;
                    else{
                        if(seenOne) save_zeros(s,zeroCount);
                        zeroCount=0;
                        seenOne=true;
                        fputc(pl[i],s);
                    }
                }
            }
            seenOne=true;
            for(int i=0;i<(int)pr.size();i++){
                if(pr[i]==info.zero) zeroCount++;
                else{
                    if(seenOne) save_zeros(s,zeroCount);
                    zeroCount=0;
                    seenOne=true;
                    fputc(pr[i],s);
                }
            }
            //if(!seenOne) fputc(info.zero,s);
        }
        fputc('\n',s);
    }
    if(info.output&OUTPUT_STATE){
        fprintf(s,"%s\n",state[info.state].c_str());
    }
    if(info.output&OUTPUT_COUNT){
        int n=0;
        for(int i=0;i<(int)pr.size();i++) if(pr[i]==info.one) n++;
        for(int i=0;i<(int)pl.size();i++) if(pl[i]==info.one) n++;
        fprintf(s,"%d\n",n);
    }
    if(info.output&OUTPUT_POS){
        fprintf(s,"%d\n",info.pos);
    }
    fclose(s);

    return 0;
}

int save_tle_output(void)
{
  FILE *s;
  s=fopen(info.outfile,"w");
  if(s==0) return -1;

  fprintf(s,"<TLE>\n");

  fclose(s);

  return 0;
}

void print_output(void)
{
  printf("OK.\n");
  printf("%d steps\n",info.steps);
  printf("%d instructions\n",instr.size());
}
